from flask import request, abort

def body_content(content,*dargs,strict=False,on_error=None,objectifier=None,**dkwargs):

	if on_error is None:
		on_error = lambda :abort(415)
	if objectifier is None:
		objectifier = lambda x:x

	def __extractor(endpoint):
		def __verification(*args,**kwargs):
			data = None
			if content == "json":
				if request.json is None and strict:
					on_error()
				elif request.json is not None:
					data = request.json
			elif content == "form":
				if request.form is None and strict:
					on_error()
				elif request.form is not None:
					data = dict(request.form)
			return endpoint(objectifier(data,*dargs,**dkwargs),*args,**kwargs)

		__verification.__name__ = endpoint.__name__
		return __verification

	return __extractor


from temod.base.attribute import StringAttribute
from temod.storage.directory import YamlStorage
from temod.base.entity import Entity

from .exceptions import *
from .utils import *



class AccessRight(Entity):
	"""docstring for AccessRight"""
	ENTITY_NAME = "right"
	ATTRIBUTES = {}
	def __init__(self, label, **kwargs):
		super(AccessRight, self).__init__(StringAttribute("label",value=label,force_lower_case=False,is_nullable=False,non_empty=True))
		self.infos = kwargs

	def __getitem__(self,attr):
		try:
			return self.infos[attr]
		except:
			pass
		return super(AccessRight,self).__getitem__(attr)



class LawBook(object):
	"""docstring for LawBook"""
	def __init__(self, rights):
		super(LawBook, self).__init__()
		self.rights = {}
		for right in rights:
			self.add_right(right)

	def add_right(self,right):
		try_assert(
			issubclass(type(right),AccessRight),MalformedLawBookError,"Added rights must be subclass of AccessRight"
		)
		try_assert(
			not (right['label'] in self.rights),MalformedLawBookError,f"Right with label '{right['label']}' is already registered"
		)
		self.rights[right['label']] = right

	def list_rights(self):
		return self.rights.values()

	def to_dict(self):
		return {
			label:right.infos for label,right in self.rights.items()
		}



class YamlLawBookKeeper(YamlStorage):
	"""docstring for YamlLawBookKeeper"""
	def __init__(self,filename, directory, encoding="utf-8", final=True):
		super(YamlLawBookKeeper, self).__init__(
			directory,encoding=encoding
		)
		self.filename = filename
		self.final = final

	def load(self):
		loaded = super(YamlLawBookKeeper,self).load(self.filename)
		if type(loaded) is list:
			self.lawBook = LawBook([
				AccessRight(element.pop('label'),**element) for element in loaded
			])
		elif type(loaded) is dict:
			self.lawBook = LawBook([
				AccessRight(label,**element) for label,element in loaded.items()
			])
		else:
			raise MalformedLawBookError("Stored access rights must be in the format list of dictionnaries or a dictionnary")
		return self.lawBook
		
	def save(self):
		if self.final:
			raise ImmutableLawBookError("Cannot overwrite saved access rights")
		return super(YamlLawBookKeeper,self).save(self.filename,self.lawBook.to_dict())


class MalformedLawBookError(Exception):
	"""docstring for MalformedLawBookError"""
	def __init__(self, *args, **kwargs):
		super(MalformedLawBookError, self).__init__(*args, **kwargs)

class ImmutableLawBookError(Exception):
	"""docstring for ImmutableLawBookError"""
	def __init__(self, *args, **kwargs):
		super(ImmutableLawBookError, self).__init__(*args, **kwargs)

class MalformedGardianError(Exception):
	"""docstring for MalformedGardianError"""
	def __init__(self, *args, **kwargs):
		super(MalformedGardianError, self).__init__(*args, **kwargs)

class LockException(Exception):
	"""docstring for LockException"""
	def __init__(self, *args, **kwargs):
		super(LockException, self).__init__(*args, **kwargs)

class MalformedAlarmError(Exception):
	"""docstring for MalformedAlarmError"""
	def __init__(self, *args, **kwargs):
		super(MalformedAlarmError, self).__init__(*args, **kwargs)
		


from temod.base import Entity, Join, Cluster 
from temod.storage import REGISTRED_STORAGES
from temod.base.attribute import UUID4Attribute

from datetime import datetime
from uuid import uuid4

from .exceptions import *



def TemodDefaultTokenGenerator():
	return {"token":UUID4Attribute.generate_random_value(), "expiration_date":None}
		

class TemodUserHandler(object):
	"""docstring for TemodUserHandler"""
	def __init__(self, user_class, database_type, identifier="id", logins=None, is_authenticated_attr="is_authenticated", is_active_attr= "is_active", 
		token_attr="token", token_expiration_attr="expiration_date", token_generator=TemodDefaultTokenGenerator, **db_credentials):
		super(TemodUserHandler, self).__init__()
		self.user_class = user_class
		self.is_authenticated_attr = is_authenticated_attr
		self.is_active_attr = is_active_attr
		self.token_attr = token_attr
		self.token_expiration_attr = token_expiration_attr
		self.token_generator = token_generator
		self.db_credentials = db_credentials
		self.identifier = identifier
		self.logins = [] if logins is None else logins
		try:
			if issubclass(user_class,Entity):
				self.database = REGISTRED_STORAGES[database_type][Entity]
			elif issubclass(user_class,Join):
				self.database = REGISTRED_STORAGES[database_type][Join]
			elif issubclass(user_class,Cluster):
				self.database = REGISTRED_STORAGES[database_type][Cluster]
			else:
				raise
		except:
			raise Exception(f"Cannot pick the right database for user class {user_class} and database {database}")

	def load_user(self,x):
		dct = {self.identifier:x}
		user = self.database(self.user_class,**self.db_credentials).get(**dct)
		if user is not None:
			return TemodUser(user,identifier=self.identifier)

	def load_user_by_token(self,token):
		dct = {self.token_attr:token}
		user = self.database(self.user_class,**self.db_credentials).get(**dct)
		if user is not None:
			if user[self.token_expiration_attr] is None or user[self.token_expiration_attr] > datetime.now():
				user[self.is_authenticated_attr] = True
				user[self.is_active_attr] = True
				return TemodUser(user,identifier=self.identifier)
			user.takeSnapshot()
			user[self.token_attr] = None
			user[self.token_expiration_attr] = None
			self.database(self.user_class,**self.db_credentials).updateOnSnapshot(user)
		return None

	def search_user(self,*logins):
		if len(logins) > len(self.logins):
			raise TemodLoginsException("There is more logins than expected")
		elif len(logins) < len(self.logins):
			raise TemodLoginsException("Some logins are missing")
		user = self.database(self.user_class,**self.db_credentials).get(
			**{self.logins[i]:logins[i] for i in range(len(logins))}
		)
		if user is not None:
			return TemodUser(user,identifier=self.identifier)

	def login_user(self,temod_user):
		temod_user.user.takeSnapshot()
		temod_user[self.is_authenticated_attr] = True
		temod_user[self.is_active_attr] = True
		return self.database(self.user_class,**self.db_credentials).updateOnSnapshot(temod_user.user)

	def logout_user(self,temod_user):
		temod_user.user.takeSnapshot()
		temod_user[self.is_authenticated_attr] = False
		temod_user[self.is_active_attr] = False
		return self.database(self.user_class,**self.db_credentials).updateOnSnapshot(temod_user.user)

	def generate_token(self,temod_user):
		temod_user.user.takeSnapshot()
		generated = self.token_generator()
		temod_user[self.token_attr] = generated[self.token_attr]
		temod_user[self.token_expiration_attr] = generated[self.token_expiration_attr]
		return self.database(self.user_class,**self.db_credentials).updateOnSnapshot(temod_user.user)



class TemodUser(object):
	"""docstring for TemodUser"""
	def __init__(self,user,identifier="id"):
		super(TemodUser, self).__init__()
		self.identifier = identifier
		self.user = user

	def __getattribute__(self,name):
		if name != "user":
			try:
				return self.user[name]
			except:
				pass
		try:
			return super(TemodUser,self).__getattribute__(name)
		except:
			return getattr(self.user,name)

	def get_id(self):
		return self.user[self.identifier]

	@property
	def is_anonymous(self):
		return self.user is None

	def __getitem__(self,name):
		return self.user[name]

	def __setitem__(self,name,value):
		self.user[name] = value


from temod_forms.readers.dict_reader import *
from temod.base.attribute import *

class FormReadersException(Exception):
	pass


class FormReadersHolder(object):
	"""docstring for FormReadersHolder"""
	def __init__(self):
		super(FormReadersHolder, self).__init__()
		self.entities = {}
		self.joins = {}
		self.clusters = {}

	def addEntity(self,entity):
		self.entities[entity.__name__] = EntityDictReader(entity)

	def addJoin(self,join):
		self.joins[join.__name__] = JoinDictReader(join)

	def addCluster(self,cluster):
		self.clusters[cluster.__name__] = ClusterDictReader(cluster)

	##########################################

	def single(self,name, **z):
		if name in self.entities and not "date_fields" in z:
			z["date_fields"] = [
				attr['name'] for attr in self[name].entity_type.ATTRIBUTES 
				if attr['type'] in [DateAttribute,DateTimeAttribute]
			]
		return lambda *x,**y: self[name].read(*x,**y, **z)

	##########################################

	def __getitem__(self,name):
		obj_ = self.entities.get(
			name,self.joins.get(
				name,self.clusters.get(name,None)
			)
		)
		if obj_ is not None:
			return obj_
		raise FormReadersException(f"Unknown object {name}.")


_readers_holder = FormReadersHolder()
		


from flask_login import LoginManager, login_user, logout_user
from flask import request, redirect, abort

from oauthlib.oauth2 import WebApplicationClient
from functools import partial
import requests
import json

from .exceptions import *


class Authenticator(LoginManager):
	"""docstring for Authenticator"""
	def __init__(self, loader=None, postload=None, login_view="login", **kwargs):
		super(Authenticator, self).__init__(**kwargs)
		self.login_view = login_view
		if loader is not None:
			self.setLoader(loader)
		self.postload = None;
		if postload is not None:
			self.setPostLoader(postload)

	def load_user(self,identifier):
		return self.loader.load_user(identifier)

	def search_user(self,*logins):
		return self.loader.search_user(*logins)

	def login_user(self,user, **kwargs):
		self.loader.login_user(user)
		return login_user(user, **kwargs)

	def logout_user(self,user):
		self.loader.logout_user(user)
		return logout_user()

	def setLoader(self,loader):
		self.loader = loader
		@self.user_loader
		def load(userID):
			loaded_user = self.loader.load_user(userID)
			if loaded_user is None or self.postload is None:
				return loaded_user
			return self.postload(loaded_user)

	def setPostLoader(self,post_loader):
		try:
			assert(hasattr(post_loader,"__call__"))
		except:
			raise MalformedAuthenticatorError("The user post loader must be a callable object.")
		self.postload = post_loader



class AuthorizationBearerAuthenticator(Authenticator):
	"""docstring for AuthorizationBearerAuthenticator"""

	def login_user(self,user):
		return self.loader.generate_token(user)

	def setLoader(self,loader):
		def load(request):
			auth_header = [v for k,v in request.headers.items() if k == "Authorization"]
			if len(auth_header) != 1 or not('Bearer ' in auth_header[0]):
				return None
			token = auth_header[0].split('Bearer ')[1]
			loaded_user = self.loader.load_user_by_token(token)
			if loaded_user is None or self.postload is None:
				return loaded_user
			return self.postload(loaded_user)

		self.loader = loader
		self._request_callback = load
		self.unauthorized_callback = lambda :abort(403)
		


class OAuth2WebAuthenticator(Authenticator):
	"""docstring for OAuth2WebAuthenticator"""
	def __init__(self, application_id, application_secret, discovery_url, *args, default_scope = None, user_class = None,
			configs_authorization_edp = "authorization_endpoint", configs_token_edp = "token_endpoint", configs_infos_edp="userinfo_endpoint",
			**kwargs
		):
		super(OAuth2WebAuthenticator, self).__init__(user_class,*args, **kwargs)

		self.application_id = application_id
		self.application_secret = application_secret
		self.discovery_url = discovery_url
		self.configs_authorization_edp = configs_authorization_edp
		self.configs_token_edp = configs_token_edp
		self.configs_infos_edp = configs_infos_edp
		self.default_scope = default_scope

		self.client = WebApplicationClient(application_id)

	def getAuthentificatorConfig(self,field=None):
		dct = requests.get(self.discovery_url).json()
		if field is None:
			return dct
		return dct[field]

	def create_authentification_endpoint(self,name,scope,callback_address=None):

		def endpointFunction(host,route):
			authorization_endpoint = self.getAuthentificatorConfig(field=self.configs_authorization_edp)
			if host is None:
				host = f"https://{request.environ['HTTP_HOST']}"
			request_uri = self.client.prepare_request_uri(
				authorization_endpoint,
				redirect_uri=host+route,
				scope=scope,
			)
			return redirect(request_uri)

		function = partial(endpointFunction,*callback_address)
		function.__name__ = name+"_auth"
		return function

	def create_callback_endpoint(self,name,loader=None):
		def endpointFunction():
			code = request.args.get("code")
			token_endpoint = self.getAuthentificatorConfig(field=self.configs_token_edp)
			token_url, headers, body = self.client.prepare_token_request(
				token_endpoint,
				authorization_response=request.url,redirect_url=request.base_url,code=code
			)
			token_response = requests.post(
				token_url,headers=headers,data=body,
				auth=(self.application_id, self.application_secret),
			)

			self.client.parse_request_body_response(json.dumps(token_response.json()))
			userinfo_endpoint = self.getAuthentificatorConfig(field=self.configs_infos_edp)
			uri, headers, body = self.client.add_token(userinfo_endpoint)
			userinfos = requests.get(uri, headers=headers, data=body).json()

			if loader is None:
				return self.loading_function(userinfos)
			return loader(userinfos)

		endpointFunction.__name__ = name+"_callback"
		return endpointFunction

	def OAUTH2endpoint(self,app,label,route,callback_route=None,callback_host=None,scope=None):
		if scope is None:
			scope = self.default_scope
		if callback_route is None:
			callback_route = f"{route}/callback"
		def __auth(function):
			app.route(f'{route}')(self.create_authentification_endpoint(label,scope,callback_address=(callback_host,callback_route)))
			app.route(f"{callback_route}")(self.create_callback_endpoint(label,loader=function))
		return __auth

	def init_app(self,app,routes=None):
		if routes is None:
			routes = []

		for route in routes:

			if not "callback" in route:
				route['callback'] = f"{route['url']}/callback"

			app.route(f'{route["url"]}')(
				self.create_authentification_endpoint(route['name'],route.get('scope',self.default_scope),route["callback"])
			)

			app.route(f"{route['callback']}")(
				self.create_callback_endpoint(route['name'],route['callback'],loader=route.get('user_loader',None))
			)
			


from pythonjsonlogger import jsonlogger
from pprint import pprint

from .exceptions import *
from .utils import *

import logging.handlers as handlers
import logging


class Alarm(object):
	"""docstring for Alarm"""
	def __init__(self,name=""):
		super(Alarm, self).__init__()
		self.name = name

	def trigger(self,**data):
		print(f"Alarm {self.name} triggered: ")
		pprint(data)


class TimedRotatingJsonFileAlarm(Alarm):
	"""docstring for TimedRotatingJsonFileAlarm"""
	def __init__(self, filename, logger_name=None, when="D", interval=1, encoding="utf-8", backupCount=0, delay=True, 
		utc=False, atTime=None, errors=None, default_state='info', level=logging.WARNING):
		super(TimedRotatingJsonFileAlarm, self).__init__(name=filename)
		try_assert(
			default_state in ['debug','info','warning','error','critical'],MalformedAlarmError,
			"Alarm default state must be one of the following: debug, info, warning, error, critical"
		)

		self.default_state = default_state

		if logger_name is None:
			self.logger = logging.getLogger()
		else:
			self.logger = logging.getLogger(logger_name)
		self.logger.setLevel(level)

		logHandler = handlers.TimedRotatingFileHandler(
			self.name,when=when,interval=interval,encoding=encoding,backupCount=backupCount,delay=delay,
			utc=False,atTime=atTime,errors=errors
		)

		formatter = jsonlogger.JsonFormatter()
		logHandler.setFormatter(formatter)
		self.logger.addHandler(logHandler)

	def trigger(self,**data):
		level = self.default_state
		if data.get('level',None) in ['debug','info','warning','error','critical']:
			level = data.pop('level')
		getattr(self.logger,level)(data)


from .exceptions import *
from .alarm import Alarm
from .right import *
from .utils import *

from datetime import datetime


class Gardian(object):
	"""docstring for Gardian"""
	def __init__(self, decision_function=lambda:True, alarm=None,strict=False):
		super(Gardian, self).__init__()
		if alarm is not None:
			try_assert(
				issubclass(type(alarm),Alarm),MalformedGardianError,"Gardian's alarm must be a subclass of Alarm."
			)
		else:
			print(f"Warning: The gardian has no alarm to trigger in case of unwanted actions.")
		self.decision_function = decision_function
		self.post_alarms = {"default":lambda:None}
		self.security_gates = {}
		self.strict = strict
		self.alarm = alarm

	def add_security_gate(self,function_,gate_id=None):
		id_ = gate_id if gate_id is not None else function_.__name__
		if id_ in self.security_gates:
			if self.strict:
				raise DuplicatedLockError(f"A function with name '{id_}' is already beeing locked by the gardian.")
			print(f"Warning: A function with name '{id_}' is already beeing locked by the gardian.")
		self.security_gates[id_]=function_
		return id_

	def decision(self,f):
		self.decision_function = f

	def alarm_response(self,*levels):
		try_assert(
			all([level in ['default','debug','info','warning','error','critical'] for level in levels]),MalformedGardianError,
			"Alarm level must be one of the following: default, debug, info, warning, error, critical"
		)
		def __response(f):
			for level in levels:
				self.post_alarms[level] = f
		return __response

	def lock_function(self, *dargs, lock_name=None, alarm_snapshot=None, trigger_alarm=True, post_alarm=None, endpoint_kwargs=None, **dkwargs):

		def __lock(locked_function):
			id_ = self.add_security_gate(locked_function,gate_id=lock_name)

			def __verification(*args,**kwargs):
				extras = {} if endpoint_kwargs is None else {k:kwargs.get(k,None) for k in endpoint_kwargs}
				decision = self.decision_function(*dargs,**dkwargs,**extras)
				if type(decision) is bool:
					if decision:
						return locked_function(*args,**{k:v for k,v in kwargs.items() if not k in extras})
				elif type(decision) is dict:
					return locked_function(*args,**{k:v for k,v in kwargs.items() if not k in extras},**decision)

				alert_level = 'default'
				if trigger_alarm and self.alarm is not None:
					data = {"at_function":id_,"time":datetime.now()}
					if alarm_snapshot is not None:
						data.update(alarm_snapshot())
					alert_level = data.get('level','default')
					self.alarm.trigger(**data)
				if post_alarm is not None:
					return post_alarm()
				return self.post_alarms.get(alert_level,'default')()

			__verification.__name__ = id_
			return __verification

		return __lock



class LawBookGardian(Gardian):
	"""docstring for LawBookGardian"""

	DEFAULT_USER_LOADER = None

	def __init__(self, lawBook, userRightsReader, index=None, **kwargs):
		super(LawBookGardian, self).__init__(
			decision_function= lambda x,y,**z: self.has_rights(x(),y,**z) ,
			**kwargs
		)

		try_assert(
			issubclass(type(lawBook),LawBook),MalformedGardianError,"LawBookGardian's law book must be a subclass of LawBook."
		)
		try_assert(
			hasattr(userRightsReader,"__call__"),MalformedGardianError,"LawBookGardian's verification process must be a callable object."
		)
		try_assert(
			hasattr(index,"__call__"),MalformedGardianError,"LawBookGardian's rights index must be a callable object."
		)

		if index is not None:
			try_assert(
				hasattr(index,"__call__"),MalformedGardianError,"LawBookGardian's rights index must be a callable object."
			)
			try_assert(
				len(set([index(right) for right in lawBook.list_rights()])) == len(lawBook.rights),MalformedGardianError,
				"LawBookGardian's rights index must keep the unicity of every right in the law book."
			)

		self.userRightsReader = userRightsReader
		self.lawBook = lawBook
		self.index = index

	def rights(self):
		return self.lawBook.list_rights()
			
	def has_right(self,user,right):
		if self.index is None:
			idx = right
		else:
			idx = self.index(self.lawBook.rights[right])
		return idx in self.userRightsReader(user)

	def has_rights(self, user, rights, mode="any"):
		if mode == 'any' and any([self.has_right(user,right) for right in rights]):
			return True
		if mode == "all" and all([self.has_right(user,right) for right in rights]):
			return True
		return False

	def lock_function(self,rights, user_loader=None, mode='any',**kwargs):

		if user_loader is None:
			user_loader = lambda: LawBookGardian.DEFAULT_USER_LOADER()

		return super(LawBookGardian,self).lock_function(
			user_loader,rights,mode=mode,**kwargs
		)

		


from flask import Blueprint as flaskBlueprint, current_app, request, session
from .exceptions import *


class Blueprint(flaskBlueprint):
	"""docstring for Blueprint"""
	def __init__(self, *args, default_config=None, **kwargs):
		super(Blueprint, self).__init__(*args, **kwargs)
		self.default_configuration = {} if default_config is None else default_config
		self.default_config(self.default_configuration)

	def default_config(self, configuration):
		try:
			self.default_configuration = dict(configuration)
		except:
			raise WrongConfigurationFormat("Blueprint configuration must be a dictionnary or an object transformable into a dictionnary using dict()")
		return self

	def setup(self, configuration):
		default = dict(self.default_configuration)
		self.configuration = {
			key: configuration.get(key, value) for key,value in default.items()
		}
		self.configuration.update(configuration)
		return self

	def get_configuration(self, config, *args, fetch_from_app=True):
		has_default = False
		if( len(args) > 0 ):
			has_default = True
			default = args[0]

		try:
			return self.configuration[config]
		except:
			if not hasattr(self,"configuration"):
				self.setup({})
				return self.get_configuration(config)
			try:
				if fetch_from_app:
					return getattr(current_app, 'config', {})[config]
			except:
				if has_default:
					return default
			raise ConfigNotFound(f"Nor the blueprint nor the app have the config: {config}")


class MultiLanguageBlueprint(Blueprint):
	"""docstring for MultiLanguageBlueprint"""

	LANGUAGES_KEY = "LANGUAGES"
	DICTIONNARY_KEY = "DICTIONNARY"

	def __init__(self, *args, language_param='lg', default_language=None, default_language_picker=None, on_language_change=None, dictionnary_selector=None,
		**kwargs):
		super(MultiLanguageBlueprint, self).__init__(*args, **kwargs)
		self.language_param = language_param
		self.default_language = default_language
		self.on_language_change = lambda x:x if on_language_change is None else on_language_change
		if not hasattr(self.on_language_change,'__call__'):
			raise CallbackException("on_language_change needs to be a callable object that takes one str argument")
		self.default_language_picker = lambda :self.default_language if default_language_picker is None else default_language_picker
		if not hasattr(self.default_language_picker,'__call__'):
			raise CallbackException("default_language_picker needs to be a callable object that takes no argument")
		self.dictionnary_selector = (lambda x:x) if dictionnary_selector is None else dictionnary_selector
		if not hasattr(self.dictionnary_selector,'__call__'):
			raise CallbackException("dictionnary_selector needs to be a callable object that takes one argument")

	def setup(self, configuration):
		if "language_param" in configuration:
			self.language_param = configuration["language_param"]
		if "default_language" in configuration:
			self.default_language = configuration["default_language"]
		return super(MultiLanguageBlueprint, self).setup(configuration)

	def _get_str_language(self, return_language_only=True):
		lg = request.args.get(self.language_param)
		is_default = False
		has_changed = False

		if lg is None:
			lg = session.get(self.language_param)
			if lg is None:
				lg = self.default_language_picker()
				if lg is None:
					try:
						lg = self.get_configuration("default_language",None)
					except:
						pass
				is_default = True
		else:
			session[self.language_param] = lg
			has_changed = True

		if not return_language_only:
			return lg, is_default, has_changed
		return lg

	def get_language(self):
		lg, is_default, has_changed = self._get_str_language(return_language_only=False)

		languages = self.get_configuration(MultiLanguageBlueprint.LANGUAGES_KEY,None)
		if languages is None:
			if has_changed:
				self.on_language_change(lg)
			return lg

		try:
			language = languages[lg]
			if has_changed:
				self.on_language_change(language)
			return language
		except:
			if not is_default:
				lg = self.default_language_picker()
				return languages[lg]
			raise LanguageNotFound("The default language was not properly set")

	def get_dictionnary(self, return_dictionnary_only=True):
		language = self.get_language()
		dictionnary = self.get_configuration(MultiLanguageBlueprint.DICTIONNARY_KEY).get(
			self.dictionnary_selector(language)
		)
		if return_dictionnary_only:
			return dictionnary
		return language, dictionnary

	def with_language(self, f):
		def __load_language(*args, **kwargs):
			return f(self.get_language(),*args, **kwargs)

		__load_language.__name__ = f.__name__
		return __load_language

	def with_dictionnary(self, f):
		def __load_dictionnary(*args, **kwargs):
			return f(*self.get_dictionnary(return_dictionnary_only=False),*args, **kwargs)

		__load_dictionnary.__name__ = f.__name__
		return __load_dictionnary
